/*
 * ringFinder.h
 *
 *  Created on: 08.04.2018
 *      Author: gevorkov
 */

#ifndef INCLUDE_RINGFINDER_H_
#define INCLUDE_RINGFINDER_H_

#include <stdint.h>
#include <vector>
#include "fastDiffractionImageProcessing/detectorGeometry.h"
#include "fastDiffractionImageProcessing/detectorRawFormat.h"
#include <Eigen/Dense>
#include <Eigen/StdVector>

typedef struct {
    float relaltiveRingDiff;
    uint16_t smoothWindowRadius;   
    uint16_t ringDilationRadius;
    uint16_t minRingRadius;
    uint16_t maxRingRadius;
} ringFinder_accuracyConstants_t;

typedef struct {
    std::vector< uint16_t > binIndices;

    std::vector< std::vector< uint32_t > > dataIndicesInBins;

    Eigen::ArrayXf bins;
    Eigen::ArrayXf dataCountInBins;

    Eigen::ArrayXf medianFilteredBins;

    detectorRawFormat_t detectorRawFormat;
    ringFinder_accuracyConstants_t accuracyConstants;
} ringFinder_precomputedConstants_t;

void precomputeRingFinderConstants(ringFinder_precomputedConstants_t& precomputedConstants, const float* detectorGeometryRadiusMatrix_linear,
        const ringFinder_accuracyConstants_t& accuracyConstants, const detectorRawFormat_t& detectorRawFormat);

void ringFinder(float* data, ringFinder_precomputedConstants_t& precomputedConstants);

#endif /* INCLUDE_RINGFINDER_H_ */
