/*
 * radialBackgroundSubtraction_wrapper.cpp
 *
 *  Created on: 04.08.2016
 *      Author: gevorkov
 */

#include "fastDiffractionImageProcessing/adaptions/cheetah/cheetahConversion.h"
#include "fastDiffractionImageProcessing/adaptions/onda/radialBackgroundSubtraction_wrapper.h"
#include "fastDiffractionImageProcessing/detectorGeometry.h"

#include <Eigen/Dense>
#include "fastDiffractionImageProcessing/eigenSTLContainers.h"
#include <cstdint>
#include <vector>

using namespace std;
using namespace Eigen;

radialBackgroundSubtraction_constantArguments_t precomputeRadialBackgroundSubtractionConstantArguments(const uint8_t* mask, const float* detectorGeometryRadiusMatrix,
        const detectorRawFormat_t& detectorRawFormat, const radialBackgroundSubtraction_accuracyConstants_pythonWrapper_t& accuracyConstants_pythonWrapper,
        detectorGeometryMatrix_pythonWrapper_t detectorGeometryMatrix_pythonWrapper)
{
    const radialBackgroundSubtraction_accuracyConstants_pythonWrapper_t& ac = accuracyConstants_pythonWrapper;

    radialBackgroundSubtraction_accuracyConstants_t accuracyConstants;

    accuracyConstants.minValuesPerBin = ac.minValuesPerBin;
    accuracyConstants.minBinWidth = ac.minBinWidth;
    accuracyConstants.maxConsideredValuesPerBin = ac.maxConsideredValuesPerBin;
    accuracyConstants.rank = ac.rank;
    accuracyConstants.rankSmoothing = ac.rankSmoothing;

    for (uint16_t i = 0; i < ac.detectorsToConsiderCount; ++i) {
        accuracyConstants.detektorsToConsiderSubscripts.emplace_back(ac.detectorsToConsiderSubscripts_x[i], ac.detectorsToConsiderSubscripts_y[i]);
    }

    for (uint16_t i = 0; i < ac.detectorsToCorrectCount; ++i) {
        accuracyConstants.detektorsToCorrectSubscripts.emplace_back(ac.detectorsToCorrectSubscripts_x[i], ac.detectorsToCorrectSubscripts_y[i]);
    }

    detectorRawFormat_t* detectorRawFormat_heap = new detectorRawFormat_t;
    *detectorRawFormat_heap = detectorRawFormat;

    vector< EigenSTL::vector_detectorPosition_t > *detectorPositions_heap = new vector< EigenSTL::vector_detectorPosition_t >;
    Vector2f* detectorGeometryMatrix;
    getDetectorGeometryMatrixFromPixelMaps(detectorGeometryMatrix_pythonWrapper.detectorGeometryMatrix_x,
            detectorGeometryMatrix_pythonWrapper.detectorGeometryMatrix_y,
            detectorRawFormat, &detectorGeometryMatrix);
    computeDetectorPositionsFromDetectorGeometryMatrix(*detectorPositions_heap, detectorRawFormat, detectorGeometryMatrix);

    radialBackgroundSubtraction_precomputedConstants_t* precomputedConstants_heap = new radialBackgroundSubtraction_precomputedConstants_t;

    precomputeBackgroundSubtractionConstants(*precomputedConstants_heap, mask, detectorGeometryRadiusMatrix, *detectorPositions_heap, detectorRawFormat,
            accuracyConstants, detectorGeometryMatrix);

    deleteDetectorGeometryMatrixMadeFromPixelMaps(detectorGeometryMatrix);

    radialBackgroundSubtraction_constantArguments_t radialRankFilter_constantArguments;
    radialRankFilter_constantArguments.precomputedConstants = precomputedConstants_heap;
    radialRankFilter_constantArguments.detectorRawFormat = detectorRawFormat_heap;
    radialRankFilter_constantArguments.detectorPositions = detectorPositions_heap;

    return radialRankFilter_constantArguments;
}

void applyRadialBackgroundSubtraction(float* data, radialBackgroundSubtraction_constantArguments_t radialRankFilter_constantArguments)
{
    applyRadialBackgroundSubtraction(data,
            *radialRankFilter_constantArguments.precomputedConstants,
            *radialRankFilter_constantArguments.detectorRawFormat,
            *radialRankFilter_constantArguments.detectorPositions);
}

void freePrecomputeRadialBackgroundSubtractionConstants(radialBackgroundSubtraction_constantArguments_t radialRankFilter_constantArguments)
{
    delete radialRankFilter_constantArguments.precomputedConstants;
    delete radialRankFilter_constantArguments.detectorRawFormat;
    delete radialRankFilter_constantArguments.detectorPositions;
}
