//============================================================================
// Name        : main.cpp
// Author      : Yaro
// Version     :
// Copyright   : Your copyright notice
// Description : Hello World in C++, Ansi-style
//============================================================================

#include <peakList.h>
#include <iostream>
#include <string.h>
#include "fastDiffractionImageProcessing/testFileHandling.h"
#include "fastDiffractionImageProcessing/mask.h"
#include "fastDiffractionImageProcessing/streakFinder.h"
#include "fastDiffractionImageProcessing/detectorGeometry.h"
#include "fastDiffractionImageProcessing/detectorRawFormat.h"
#include "fastDiffractionImageProcessing/radialBackgroundSubtraction.h"
#include "fastDiffractionImageProcessing/ringFinder.h"
#include <Eigen/StdVector>

#include <stdlib.h>
#include "fastDiffractionImageProcessing/streakFinder.h"
#include <chrono>
#include "fastDiffractionImageProcessing/peakFinder9_implementation.h"

using namespace std;
using namespace Eigen;

void flushCache()
{
    const int size = 20 * 1024 * 1024; // Allocate 20M. Set much larger then L2
    volatile char *c = (char *) malloc(size);
    volatile int i = 8;
    for (volatile int j = 0; j < size; j++)
        c[j] = i * j;

    free((void*) c);
}

int main()
{

    char importImageFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\image";
    char importMaskFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\mask";
    char importDetectorGeometryMatrixCommonFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\detectorGeometryMatrix_float";
    char detectorRawSizeFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\detectorRawSize";

    char exportImageFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\image_processedByCpp";
    char exportMaskFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\mask_processedByCpp";

    float *data, *dataCopy;
    uint8_t* mask;
    detectorRawFormat_t detectorRawFormat;
    Vector2f* detectorGeometryMatrix;
    float* detectorGeometryRadiusMatrix;
    detectorPositions_t detectorPositions;

    peakList_t peakList;
    long NpeaksMax = 1024;

    vector< uint32_t > sparseMask;

    importTestImage(importImageFilename, importMaskFilename, importDetectorGeometryMatrixCommonFilename, detectorRawSizeFilename, &data, &mask,
            &detectorGeometryMatrix, &detectorGeometryRadiusMatrix, detectorRawFormat);
    allocateTestImageCopy(&dataCopy, detectorRawFormat);
    allocatePeakList(&peakList, NpeaksMax);

    computeDetectorPositionsFromDetectorGeometryMatrix(detectorPositions, detectorRawFormat, detectorGeometryMatrix);

    createSparseMask(mask, detectorRawFormat, sparseMask);

    ringFinder_accuracyConstants_t ringFinder_accuracyConstants;
    ringFinder_accuracyConstants.relaltiveRingDiff = 0.3;
    ringFinder_accuracyConstants.smoothWindowRadius = 15;
    ringFinder_accuracyConstants.ringDilationRadius = 5;
    ringFinder_accuracyConstants.minRingRadius = 100;
    ringFinder_accuracyConstants.maxRingRadius = 10000;

    ringFinder_precomputedConstants_t ringFinder_precomputedConstants;
    precomputeRingFinderConstants(ringFinder_precomputedConstants, detectorGeometryRadiusMatrix, ringFinder_accuracyConstants, detectorRawFormat);

    mergeMaskAndDataIntoDataCopy(data, dataCopy, sparseMask, detectorRawFormat);
    flushCache();
    chrono::high_resolution_clock::time_point t1 = chrono::high_resolution_clock::now();
    ringFinder(dataCopy, ringFinder_precomputedConstants);
    chrono::high_resolution_clock::time_point t2 = chrono::high_resolution_clock::now();

    auto duration = chrono::duration_cast< chrono::milliseconds >(t2 - t1).count();
    cout << "duration: " << duration << "ms" << endl;

    getMaskFromMergedMaskInData(dataCopy, mask, detectorRawFormat);
    exportTestImage(exportImageFilename, exportMaskFilename, dataCopy, mask, detectorRawFormat);

    freePeakList(peakList);
    deleteTestImageCopy(dataCopy);
    deleteTestImage(data, mask, detectorGeometryMatrix, detectorGeometryRadiusMatrix);

    return 0;
}

//radial filter + streak + peak
//int main()
//{
//
//	char importImageFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\image";
//	char importMaskFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\mask";
//	char importDetectorGeometryMatrixCommonFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\detectorGeometryMatrix_float";
//	char detectorRawSizeFilename[] = "P:\\Matlab_workspace\\data\\detectorRawSize";
//
//	char exportImageFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\image_processedByCpp";
//	char exportMaskFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\mask_processedByCpp";
//
//	float *data, *dataCopy;
//	uint8_t* mask;
//	detectorRawFormat_t detectorRawFormat;
//	Vector2f* detectorGeometryMatrix;
//	float* detectorGeometryRadiusMatrix;
//	detectorPositions_t detectorPositions;
//
//	peakList_t peakList;
//	long NpeaksMax = 1024;
//
//	vector < uint32_t > sparseMask;
//
//	importTestImage(importImageFilename, importMaskFilename, importDetectorGeometryMatrixCommonFilename, detectorRawSizeFilename, &data, &mask,
//		&detectorGeometryMatrix, &detectorGeometryRadiusMatrix, detectorRawFormat);
//	allocateTestImageCopy(&dataCopy, detectorRawFormat);
//	allocatePeakList(&peakList, NpeaksMax);
//
//	computeDetectorPositionsFromDetectorGeometryMatrix(detectorPositions, detectorRawFormat, detectorGeometryMatrix);
//
//	createSparseMask(mask, detectorRawFormat, sparseMask);
//
//	radialRankFilter_accuracyConstants_t radialRankFilter_accuracyConstants;
//	radialRankFilter_accuracyConstants.minValuesPerBin = 50;
//	radialRankFilter_accuracyConstants.minBinWidth = 3;
//	radialRankFilter_accuracyConstants.maxConsideredValuesPerBin = 500;
//	radialRankFilter_accuracyConstants.rank = 0.5;
//	for (int i = 0; i < 8; ++i) {
//		for (int j = 0; j < 8; ++j) {
//			radialRankFilter_accuracyConstants.detektorsToConsiderSubscripts.push_back(Point2D < uint8_t >(i, j));
//			radialRankFilter_accuracyConstants.detektorsToCorrectSubscripts.push_back(Point2D < uint8_t >(i, j));
//		}
//	}
//	//    radialRankFilter_accuracyConstants.detektorsToConsiderIndices.push_back(Point2D < uint8_t > (0, 1));
//	//    radialRankFilter_accuracyConstants.detektorsToCorrectIndices.push_back(Point2D < uint8_t > (0, 1));
//	//    radialRankFilter_accuracyConstants.detektorsToConsiderIndices.push_back(Point2D < uint8_t > (2, 1));
//	//    radialRankFilter_accuracyConstants.detektorsToCorrectIndices.push_back(Point2D < uint8_t > (2, 1));
//
//	radialRankFilter_precomputedConstants_t radialRankFilter_precomputedConstants;
//	precomputeRadialRankFilterConstants(radialRankFilter_precomputedConstants, mask, detectorGeometryRadiusMatrix, detectorPositions,
//		detectorRawFormat, radialRankFilter_accuracyConstants, detectorGeometryMatrix);
//
//	mergeMaskAndDataIntoDataCopy(data, dataCopy, sparseMask, detectorRawFormat);
//	flushCache();
//	chrono::high_resolution_clock::time_point t1 = chrono::high_resolution_clock::now();
//	applyRadialRankFilterBackgroundSubtraction(dataCopy, radialRankFilter_precomputedConstants, detectorRawFormat, detectorPositions);
//	chrono::high_resolution_clock::time_point t2 = chrono::high_resolution_clock::now();
//
//	auto duration = chrono::duration_cast < chrono::milliseconds > (t2 - t1).count();
//	cout << "duration: " << duration << "ms" << endl;
//
//	getMaskFromMergedMaskInData(dataCopy, mask, detectorRawFormat);
//	exportTestImage(exportImageFilename, exportMaskFilename, dataCopy, mask, detectorRawFormat);
//
//	freePeakList(peakList);
//	deleteTestImageCopy(dataCopy);
//	deleteTestImage(data, mask, detectorGeometryMatrix, detectorGeometryRadiusMatrix);
//
//	return 0;
//}

//int main()
//{
//    char importImageFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\image";
//    char importMaskFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\mask";
//    char importDetectorGeometryMatrixCommonFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\detectorGeometryMatrix_float";
//    char importDetectorRawSizeFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\detectorRawSize";
//
//    char exportImageFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\image_processedByCpp";
//    char exportMaskFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\mask_processedByCpp";
//
//    float *data, *dataCopy;
//    uint8_t* mask;
//    detectorRawFormat_t detectorRawFormat;
//    Vector2f* detectorGeometryMatrix;
//    float* detectorGeometryRadiusMatrix;
//    detectorPositions_t detectorPositions;
//
//    peakList_t peakList;
//    long NpeaksMax = 1024;
//
//    vector< uint32_t > sparseMask;
//
//    importTestImage(importImageFilename, importMaskFilename, importDetectorGeometryMatrixCommonFilename, importDetectorRawSizeFilename, &data, &mask,
//            &detectorGeometryMatrix, &detectorGeometryRadiusMatrix, detectorRawFormat);
//    allocateTestImageCopy(&dataCopy, detectorRawFormat);
//    allocatePeakList(&peakList, NpeaksMax);
//
//    computeDetectorPositionsFromDetectorGeometryMatrix(detectorPositions, detectorRawFormat, detectorGeometryMatrix);
//
//    createSparseMask(mask, detectorRawFormat, sparseMask);
//
//    peakFinder9_accuracyConstants_t peakFinder9_accuracyConstants;
//    peakFinder9_accuracyConstants.minSNR_biggestPixel = 7;
//    peakFinder9_accuracyConstants.minSNR_peakPixel = 6;
//    peakFinder9_accuracyConstants.minSNR_wholePeak = 9;
//    peakFinder9_accuracyConstants.minimumSigma = 11;
//    peakFinder9_accuracyConstants.minimumPeakOversizeOverNeighbours = 50;
//    peakFinder9_accuracyConstants.windowRadius = 3;
//
//    mergeMaskAndDataIntoDataCopy(data, dataCopy, sparseMask, detectorRawFormat);
//
//    flushCache();
//
//    chrono::high_resolution_clock::time_point t1 = chrono::high_resolution_clock::now();
//
//    peakFinder9(dataCopy, peakFinder9_accuracyConstants, detectorRawFormat, peakList);
//
//    chrono::high_resolution_clock::time_point t2 = chrono::high_resolution_clock::now();
//
//    auto duration = chrono::duration_cast< chrono::milliseconds >(t2 - t1).count();
//    cout << "duration: " << duration << "ms" << endl;
//    cout << "peak count: " << peakList.peakCount << endl << endl;
//
//    getMaskFromMergedMaskInData(dataCopy, mask, detectorRawFormat);
//    exportTestImage(exportImageFilename, exportMaskFilename, dataCopy, mask, detectorRawFormat);
//
//    freePeakList(peakList);
//    deleteTestImageCopy(dataCopy);
//    deleteTestImage(data, mask, detectorGeometryMatrix, detectorGeometryRadiusMatrix);
//
//    return 0;
//}

// //radial filter + streak + peak
//int main()
//{
//
//    char importImageFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\image";
//    char importMaskFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\mask";
//    char importDetectorGeometryMatrixCommonFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\detectorGeometryMatrix_float";
//    char detectorRawSizeFilename[] = "P:\\Matlab_workspace\\data\\detectorRawSize";
//
//    char exportImageFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\image_processedByCpp";
//    char exportMaskFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\Desy\\data\\mask_processedByCpp";
//
//    float *data, *dataCopy;
//    uint8_t* mask;
//    detectorRawFormat_t detectorRawFormat;
//    Vector2f* detectorGeometryMatrix;
//    float* detectorGeometryRadiusMatrix;
//    detectorPositions_t detectorPositions;
//
//	peakList_t peakList;
//    long NpeaksMax = 1024;
//
//    vector < uint32_t > sparseMask;
//
//    importTestImage(importImageFilename, importMaskFilename, importDetectorGeometryMatrixCommonFilename, detectorRawSizeFilename, &data, &mask,
//            &detectorGeometryMatrix, &detectorGeometryRadiusMatrix, detectorRawFormat);
//    allocateTestImageCopy(&dataCopy, detectorRawFormat);
//    allocatePeakList(&peakList, NpeaksMax);
//
//    computeDetectorPositionsFromDetectorGeometryMatrix(detectorPositions, detectorRawFormat, detectorGeometryMatrix);
//
//    createSparseMask(mask, detectorRawFormat, sparseMask);
//
//    radialRankFilter_accuracyConstants_t radialRankFilter_accuracyConstants;
//    radialRankFilter_accuracyConstants.minValuesPerBin = 50;
//    radialRankFilter_accuracyConstants.minBinWidth = 3;
//    radialRankFilter_accuracyConstants.maxConsideredValuesPerBin = 500;
//    radialRankFilter_accuracyConstants.rank = 0.5;
//    for (int i = 0; i < 8; ++i) {
//        for (int j = 0; j < 8; ++j) {
//            radialRankFilter_accuracyConstants.detektorsToConsiderSubscripts.push_back(Point2D < uint8_t > (i, j));
//            radialRankFilter_accuracyConstants.detektorsToCorrectSubscripts.push_back(Point2D < uint8_t > (i, j));
//        }
//    }
////    radialRankFilter_accuracyConstants.detektorsToConsiderIndices.push_back(Point2D < uint8_t > (0, 1));
////    radialRankFilter_accuracyConstants.detektorsToCorrectIndices.push_back(Point2D < uint8_t > (0, 1));
////    radialRankFilter_accuracyConstants.detektorsToConsiderIndices.push_back(Point2D < uint8_t > (2, 1));
////    radialRankFilter_accuracyConstants.detektorsToCorrectIndices.push_back(Point2D < uint8_t > (2, 1));
//
//    streakFinder_accuracyConstants_t streakFinder_accuracyConstants;
//    streakFinder_accuracyConstants.filterLength = 9;
//    streakFinder_accuracyConstants.minFilterLength = 5;
//    streakFinder_accuracyConstants.filterStep = 1.3;
//    streakFinder_accuracyConstants.minSNR = 9;
//    streakFinder_accuracyConstants.streakElongationMinStepsCount = 4;
//    streakFinder_accuracyConstants.streakElongationRadiusFactor = 0.01;
//    streakFinder_accuracyConstants.streakPixelMaskRadius = 2;
//    streakFinder_accuracyConstants.linesToCheck.push_back(1);
//    streakFinder_accuracyConstants.linesToCheck.push_back(2);
//    streakFinder_accuracyConstants.linesToCheck.push_back(3);
//    setStreakDetectorIndices(streakFinder_accuracyConstants, detectorCategory_t::detectorCategory_CSPAD);
//    setUserSelection_backgroundEstimationRegionInDetector(streakFinder_accuracyConstants, detectorRawFormat, 1, 10);
//
//    peakFinder9_accuracyConstants_t peakFinder9_accuracyConstants;
//    peakFinder9_accuracyConstants.minSNR_biggestPixel = 7;
//    peakFinder9_accuracyConstants.minSNR_peakPixel = 6;
//    peakFinder9_accuracyConstants.minSNR_wholePeak = 9;
//    peakFinder9_accuracyConstants.minimumSigma = 11;
//    peakFinder9_accuracyConstants.minimumPeakOversizeOverNeighbours = 50;
//    peakFinder9_accuracyConstants.windowRadius = 3;
//
//    radialRankFilter_precomputedConstants_t radialRankFilter_precomputedConstants;
//    precomputeRadialRankFilterConstants(radialRankFilter_precomputedConstants, mask, detectorGeometryRadiusMatrix, detectorPositions,
//            detectorRawFormat, radialRankFilter_accuracyConstants, detectorGeometryMatrix);
//
//    mergeMaskAndDataIntoDataCopy(data, dataCopy, sparseMask, detectorRawFormat);
//    flushCache();
//    chrono::high_resolution_clock::time_point t1 = chrono::high_resolution_clock::now();
//    applyRadialRankFilter(dataCopy, radialRankFilter_accuracyConstants, radialRankFilter_precomputedConstants, detectorRawFormat, detectorPositions);
//    chrono::high_resolution_clock::time_point t2 = chrono::high_resolution_clock::now();
//
//    streakFinder_precomputedConstants_t streakFinder_precomputedConstants;
////    precomputeStreakFinderConstants(streakFinder_accuracyConstants, detectorRawFormat, detectorPositions, mask, streakFinder_precomputedConstants);
////
////    flushCache();
////
////    chrono::high_resolution_clock::time_point t1 = chrono::high_resolution_clock::now();
////
////    mergeMaskAndDataIntoDataCopy(data, dataCopy, sparseMask, detectorRawFormat);
////    streakFinder(dataCopy, streakFinder_accuracyConstants, detectorRawFormat, detectorPositions, streakFinder_precomputedConstants);
////    peakFinder9(dataCopy, peakFinder9_accuracyConstants, detectorRawFormat, peakList);
////
////    chrono::high_resolution_clock::time_point t2 = chrono::high_resolution_clock::now();
////
//    auto duration = chrono::duration_cast < chrono::milliseconds > (t2 - t1).count();
//    cout << "duration: " << duration << "ms" << endl;
////    cout << "peak count: " << peakList.peakCount << endl << endl;
//
//    getMaskFromMergedMaskInData(dataCopy, mask, detectorRawFormat);
//    exportTestImage(exportImageFilename, exportMaskFilename, dataCopy, mask, detectorRawFormat);
//
//    freePeakList(peakList);
//    deleteTestImageCopy(dataCopy);
//    deleteTestImage(data, mask, detectorGeometryMatrix, detectorGeometryRadiusMatrix);
////    freePrecomputedStreakFinderConstants(streakFinder_precomputedConstants);
//
//    return 0;
//}

//// only streakfinder FLASH CCD
//int main()
//{
//    char importImageFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\image";
//    char importMaskFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\mask";
//    char importDetectorGeometryMatrixCommonFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\detectorGeometryMatrix_float";
//    char importDetectorRawSizeFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\detectorRawSize";
//
//    char exportImageFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\image_processedByCpp";
//    char exportMaskFilename[] = "C:\\DesyFiles\\workspaces\\Matlab_workspace\\data\\mask_processedByCpp";
//
//    float *data, *dataCopy;
//    uint8_t* mask;
//    detectorRawFormat_t detectorRawFormat;
//    Vector2f* detectorGeometryMatrix;
//    float* detectorGeometryRadiusMatrix;
//    detectorPositions_t detectorPositions;
//
//    vector< uint32_t > sparseMask;
//
//    importTestImage(importImageFilename, importMaskFilename, importDetectorGeometryMatrixCommonFilename, importDetectorRawSizeFilename, &data, &mask,
//            &detectorGeometryMatrix, &detectorGeometryRadiusMatrix, detectorRawFormat);
//    allocateTestImageCopy(&dataCopy, detectorRawFormat);
//
//    computeDetectorPositionsFromDetectorGeometryMatrix(detectorPositions, detectorRawFormat, detectorGeometryMatrix);
//
//    createSparseMask(mask, detectorRawFormat, sparseMask);
//
//    radialRankFilter_accuracyConstants_t radialRankFilter_accuracyConstants;
//    radialRankFilter_accuracyConstants.minValuesPerBin = 50;
//    radialRankFilter_accuracyConstants.minBinWidth = 3;
//    radialRankFilter_accuracyConstants.maxConsideredValuesPerBin = 500;
//    radialRankFilter_accuracyConstants.rank = 0.1;
//    radialRankFilter_accuracyConstants.detektorsToConsiderSubscripts.emplace_back(0, 0);
//    radialRankFilter_accuracyConstants.detektorsToCorrectSubscripts.emplace_back(0, 0);
//
//    streakFinder_accuracyConstants_t streakFinder_accuracyConstants;
//    streakFinder_accuracyConstants.filterLength = 20;
//    streakFinder_accuracyConstants.minFilterLength = 5;
//    streakFinder_accuracyConstants.filterStep = 1.5;
//    streakFinder_accuracyConstants.minSNR = 40;
//    streakFinder_accuracyConstants.streakElongationMinStepsCount = 25;
//    streakFinder_accuracyConstants.streakElongationRadiusFactor = 0.5;
//    streakFinder_accuracyConstants.streakPixelMaskRadius = 1;
//    streakFinder_accuracyConstants.pixelsToCheck.emplace_back(31, 67);
//    streakFinder_accuracyConstants.pixelsToCheck.emplace_back(34, 156);
//    streakFinder_accuracyConstants.pixelsToCheck.emplace_back(48, 135);
//    streakFinder_accuracyConstants.pixelsToCheck.emplace_back(55, 113);
//    streakFinder_accuracyConstants.pixelsToCheck.emplace_back(6, 149);
//    auto& backgroundRegions = streakFinder_accuracyConstants.backgroundEstimationRegions;
//    uint16_t width = 10;
//    uint16_t height = 10;
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 20), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 50), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 80), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 110), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 140), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 170), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 200), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(140, 230), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(170, 110), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(170, 140), width, height);
//    backgroundRegions.emplace_back(Point2D< uint16_t >(170, 170), width, height);
//
//    streakFinder_precomputedConstants_t streakFinder_precomputedConstants;
//    precomputeStreakFinderConstants(streakFinder_accuracyConstants, detectorRawFormat, detectorPositions, mask, streakFinder_precomputedConstants);
//
//    radialRankFilter_precomputedConstants_t radialRankFilter_precomputedConstants;
//    precomputeRadialRankFilterConstants(radialRankFilter_precomputedConstants, mask, detectorGeometryRadiusMatrix, detectorPositions,
//            detectorRawFormat, radialRankFilter_accuracyConstants, detectorGeometryMatrix);
//
//    flushCache();
//
//    chrono::high_resolution_clock::time_point t1 = chrono::high_resolution_clock::now();
//
//    mergeMaskAndDataIntoDataCopy(data, dataCopy, sparseMask, detectorRawFormat);
//    applyRadialRankFilterBackgroundSubtraction(dataCopy, radialRankFilter_precomputedConstants, detectorRawFormat, detectorPositions);
//    streakFinder(dataCopy, streakFinder_accuracyConstants, detectorRawFormat, detectorPositions, streakFinder_precomputedConstants);
//
//    chrono::high_resolution_clock::time_point t2 = chrono::high_resolution_clock::now();
//
//    auto duration = chrono::duration_cast< chrono::milliseconds >(t2 - t1).count();
//    cout << "duration: " << duration << "ms" << endl;
//
//    getMaskFromMergedMaskInData(dataCopy, mask, detectorRawFormat);
//    exportTestImage(exportImageFilename, exportMaskFilename, dataCopy, mask, detectorRawFormat);
//
//    deleteTestImageCopy(dataCopy);
//    deleteTestImage(data, mask, detectorGeometryMatrix, detectorGeometryRadiusMatrix);
//    freePrecomputedStreakFinderConstants(streakFinder_precomputedConstants);
//
//    return 0;
//}

////pnccd raw
//int main()
//{
//
////    char importImageFilename[] = "P:\\Matlab_workspace\\data\\image";
////    char importMaskFilename[] = "P:\\Matlab_workspace\\data\\mask";
////    char importDetectorGeometryMatrixCommonFilename[] = "P:\\Matlab_workspace\\data\\detectorGeometryMatrix_float";
////    char detectorRawSizeFilename[] = "P:\\Matlab_workspace\\data\\detectorRawSize";
////
////    char exportImageFilename[] = "P:\\Matlab_workspace\\data\\image_processedByCpp";
////    char exportMaskFilename[] = "P:\\Matlab_workspace\\data\\mask_processedByCpp";
//    char importImageFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\data\\image";
//    char importMaskFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\data\\mask";
//    char importDetectorGeometryMatrixCommonFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\data\\detectorGeometryMatrix_float";
//    char detectorRawSizeFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\data\\detectorRawSize";
//
//    char exportImageFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\data\\image_processedByCpp";
//    char exportMaskFilename[] = "C:\\Users\\Yaro\\Documents\\MATLAB\\data\\mask_processedByCpp";
//
//    float *data, *dataCopy, *data_rearranged;
//    uint8_t *mask, *mask_rearranged;
//    detectorRawFormat_t detectorRawFormat;
//    Vector2f* detectorGeometryMatrix;
//    float* detectorGeometryRadiusMatrix;
//    detectorPositions_t detectorPositions;
//    detectorPositions_t detectorPositions_rearranged;
//
//    tPeakList peakList;
//    long NpeaksMax = 1024;
//
//    importTestImage(importImageFilename, importMaskFilename, importDetectorGeometryMatrixCommonFilename, detectorRawSizeFilename, &data, &mask,
//            &detectorGeometryMatrix, &detectorGeometryRadiusMatrix, detectorRawFormat);
//    allocateTestImageCopy(&dataCopy, detectorRawFormat);
//    allocatePeakList(&peakList, NpeaksMax);
//
//    copy(data, data + detectorRawFormat.pix_nn, dataCopy);
//
//    computeDetectorPositionsFromDetectorGeometryMatrix(detectorPositions, detectorRawFormat, detectorGeometryMatrix);
//    rearrangePnCcdGeometryForStreakFinder(detectorPositions_rearranged, detectorPositions);
//
//    streakFinder_accuracyConstants_t streakFinder_accuracyConstants;
//    streakFinder_accuracyConstants.filterLength = 35;
//    streakFinder_accuracyConstants.minFilterLength = 20;
//    streakFinder_accuracyConstants.filterStep = 1.6;
//    streakFinder_accuracyConstants.minSNR = 17;
//    streakFinder_accuracyConstants.streakElongationMinStepsCount = 15;
//    streakFinder_accuracyConstants.streakElongationRadiusFactor = 0.08;
//    streakFinder_accuracyConstants.streakPixelMaskRadius = 2;
////    streakFinder_accuracyConstants.linesToCheck.push_back(1);
//    streakFinder_accuracyConstants.linesToCheck.push_back(3);
////    streakFinder_accuracyConstants.linesToCheck.push_back(5);
////    streakFinder_accuracyConstants.linesToCheck.push_back(8);
////    streakFinder_accuracyConstants.linesToCheck.push_back(14);
////    streakFinder_accuracyConstants.linesToCheck.push_back(24);
////    streakFinder_accuracyConstants.linesToCheck.push_back(27);
////    streakFinder_accuracyConstants.linesToCheck.push_back(35);
////    streakFinder_accuracyConstants.linesToCheck.push_back(45);
//    setStreakDetectorIndices(streakFinder_accuracyConstants, detectorCategory_t::detectorCategory_pnCCD);
//    streakFinder_accuracyConstants.backgroundEstimationRegionsInDetector.push_back(ImageRectangle < uint16_t > (Point2D < uint16_t > (100, 400), 10, 10));
//    streakFinder_accuracyConstants.backgroundEstimationRegionsInDetector.push_back(ImageRectangle < uint16_t > (Point2D < uint16_t > (250, 250), 10, 10));
//    streakFinder_accuracyConstants.backgroundEstimationRegionsInDetector.push_back(ImageRectangle < uint16_t > (Point2D < uint16_t > (450, 200), 10, 10));
//
//    mergeMaskIntoData(dataCopy, mask, detectorRawFormat);
//
////    mask_rearranged = new uint8_t[detectorRawFormat.pix_nn];
////    rearrangePnCcdMaskForStreakFinder(mask_rearranged, mask);
////    streakFinder_precomputedConstants_t streakFinder_precomputedConstants;
////    precomputeStreakFinderConstants(streakFinder_accuracyConstants, detectorRawFormat, detectorPositions_rearranged, mask_rearranged,
////            streakFinder_precomputedConstants);
//    flushCache();
//
//    chrono::high_resolution_clock::time_point t1 = chrono::high_resolution_clock::now();
//
//    pnCcdModuleWiseOrderFilterBackgroundSubtraction(dataCopy, mask);
//
////    data_rearranged = new float[detectorRawFormat.pix_nn];
////    rearrangePnCcdDataForStreakFinder(data_rearranged, dataCopy);
////    streakFinder(data_rearranged, streakFinder_accuracyConstants, detectorRawFormat, detectorPositions_rearranged, streakFinder_precomputedConstants);
////    reRearrangePnCcdDataForStreakFinder(dataCopy, data_rearranged);
//
//    chrono::high_resolution_clock::time_point t2 = chrono::high_resolution_clock::now();
//
//    auto duration = chrono::duration_cast < chrono::milliseconds > (t2 - t1).count();
//    cout << "duration: " << duration << "ms" << endl;
//
//    getMaskFromMergedMaskInData(dataCopy, mask, detectorRawFormat);
//    exportTestImage(exportImageFilename, exportMaskFilename, dataCopy, mask, detectorRawFormat);
//
//    deleteTestImageCopy(dataCopy);
//    deleteTestImage(data, mask, detectorGeometryMatrix, detectorGeometryRadiusMatrix);
////    freePrecomputedStreakFinderConstants(streakFinder_precomputedConstants);
//    delete[] mask_rearranged;
//    delete[] data_rearranged;
//
//    return 0;
//}

