/*
 * testFileHandling.cpp
 *
 *  Created on: 10.12.2015
 *      Author: Yaro
 */

#include "fastDiffractionImageProcessing/testFileHandling.h"
#include <fstream>
#include <string>
#include <iostream>
#include <sstream>
//#include <chrono>
#include <vector>
#include <limits>
//#include <math.h>
#include <cmath>
#include <cstring>

using namespace std;
using namespace Eigen;

static void loadFileIntoVector(const char fileName[], vector< char > &fileContents);
static void storeArrayInFile(const char* arr, const char fileName[], size_t size);
static float clip(float n, float lower, float upper);


void importTestImage(const char imageFilename[], const char maskFilename[], const char detectorGeometryMatrixCommonFilename[],
        const char detectorRawSizeFilename[], float** data, uint8_t** mask, Vector2f** detectorGeometryMatrix, float** detectorGeometryRadiusMatrix,
        detectorRawFormat_t& detectorRawFormat)
{
    vector< char > fileContents;
    loadFileIntoVector(detectorRawSizeFilename, fileContents);

    uint16_t asic_nx, asic_ny, nasics_x, nasics_y;
    asic_nx = (uint16_t) *((double*) (&fileContents[8 * 0]));
    asic_ny = (uint16_t) *((double*) (&fileContents[8 * 1]));
    nasics_x = (uint16_t) *((double*) (&fileContents[8 * 2]));
    nasics_y = (uint16_t) *((double*) (&fileContents[8 * 3]));

    detectorRawFormat.asic_nx = asic_nx;
    detectorRawFormat.asic_ny = asic_ny;
    detectorRawFormat.nasics_x = nasics_x;
    detectorRawFormat.nasics_y = nasics_y;

    detectorRawFormat.pix_nx = asic_nx * nasics_x;
    detectorRawFormat.pix_ny = asic_ny * nasics_y;
    detectorRawFormat.pix_nn = asic_nx * nasics_x * asic_ny * nasics_y;

    uint32_t pix_nn = detectorRawFormat.pix_nn;

    loadFileIntoVector(imageFilename, fileContents);

    *data = new float[pix_nn];
    for (uint32_t i = 0; i < pix_nn; ++i) {
        (*data)[i] = (float) *((double*) (&fileContents[8 * i]));
    }

    loadFileIntoVector(maskFilename, fileContents);

    *mask = new uint8_t[pix_nn];
    for (uint32_t i = 0; i < pix_nn; ++i) {
        (*mask)[i] = (uint8_t) *((double*) (&fileContents[8 * i]));
    }

    vector< char > fileContents_x, fileContents_y;
    vector< char > fileName_x, fileName_y;
    fileName_x.assign(detectorGeometryMatrixCommonFilename, detectorGeometryMatrixCommonFilename + strlen(detectorGeometryMatrixCommonFilename));
    fileName_y.assign(detectorGeometryMatrixCommonFilename, detectorGeometryMatrixCommonFilename + strlen(detectorGeometryMatrixCommonFilename));
    fileName_x.push_back('_');
    fileName_x.push_back('x');
    fileName_x.push_back('\0');
    fileName_y.push_back('_');
    fileName_y.push_back('y');
    fileName_y.push_back('\0');

    loadFileIntoVector(fileName_x.data(), fileContents_x);
    loadFileIntoVector(fileName_y.data(), fileContents_y);

    *detectorGeometryMatrix = new Vector2f[pix_nn];
    *detectorGeometryRadiusMatrix = new float[pix_nn];
    for (uint32_t i = 0; i < pix_nn; ++i) {
        float x = (float) *((double*) (&fileContents_x[8 * i]));
        float y = (float) *((double*) (&fileContents_y[8 * i]));

        (*detectorGeometryMatrix)[i] = Vector2f(x, y);
        (*detectorGeometryRadiusMatrix)[i] = Vector2f(x, y).norm();
    }

}

void exportTestImage(const char imageFilename[], const char maskFilename[], float* data, uint8_t* mask,
        const detectorRawFormat_t& detectorRawFormat)
{
    uint32_t pixelCount = detectorRawFormat.pix_nn;

    float dataFillValue = 0;
    vector< float > dataVector(pixelCount, dataFillValue);
    int16_t maskFillValue = 0;
    vector< int16_t > maskVector(pixelCount, maskFillValue);

    for (uint32_t i = 0; i < pixelCount; ++i) {
        if (isfinite(data[i])) {
            dataVector[i] = data[i];
        }
        if (isfinite(mask[i])) {
            maskVector[i] = (int16_t) clip(mask[i], numeric_limits< int16_t >::min(), numeric_limits< int16_t >::max());
        }
    }
    storeArrayInFile((char*) dataVector.data(), imageFilename, pixelCount * sizeof(*dataVector.data()));
    storeArrayInFile((char*) maskVector.data(), maskFilename, pixelCount * sizeof(*maskVector.data()));
}

void allocateTestImageCopy(float** dataCopy, const detectorRawFormat_t& detectorRawFormat)
{
    uint32_t imageSize_x = detectorRawFormat.asic_nx * detectorRawFormat.nasics_x;
    uint32_t imageSize_y = detectorRawFormat.asic_ny * detectorRawFormat.nasics_y;

    *dataCopy = new float[imageSize_x * imageSize_y];
    //*dataCopy = new float[detectorRawFormat.pix_nn];
}
void deleteTestImageCopy(float* dataCopy)
{
    delete[] dataCopy;
}

void deleteTestImage(float* data, uint8_t* mask, Vector2f* detectorGeometryMatrix, float* detectorGeometryRadiusMatrix)
{
    delete[] data;
    delete[] mask;
    delete[] detectorGeometryMatrix;
    delete[] detectorGeometryRadiusMatrix;
}

static float clip(float n, float lower, float upper)
{
    return std::max(lower, std::min(n, upper));
}

static void loadFileIntoVector(const char fileName[], vector< char > &fileContents)
{
    ifstream myfile(fileName, ios::binary);

    if (!myfile.is_open()) {
        cerr << "wrong input file: " << fileName;
    }

    myfile.seekg(0, std::ios::end);
    fileContents.resize(myfile.tellg());
    myfile.seekg(0, std::ios::beg);
    myfile.read(&fileContents[0], fileContents.size());

    myfile.close();
}

static void storeArrayInFile(const char* arr, const char fileName[], size_t size)
{
    ofstream myfile(fileName, ios::binary);
    myfile.write(arr, size);
    myfile.close();
}
